@@ -121,31 +121,61 @@ impl TriMeshWrapper {
         (point_to_vec(&aabb.mins), point_to_vec(&aabb.maxs))
     }
 
-    pub fn intersections(
+    pub fn intersections_many(
         &self,
-        _py: Python,
+        py: Python,
         src_points: Vec<Vec<Precision>>,
         tgt_points: Vec<Vec<Precision>>,
     ) -> (Vec<usize>, Vec<Vec<Precision>>, Vec<bool>) {
-        let mut idxs = Vec::default();
-        let mut intersections = Vec::default();
-        let mut is_backface = Vec::default();
+        py.allow_threads(|| {
+            let mut idxs = Vec::default();
+            let mut intersections = Vec::default();
+            let mut is_backface = Vec::default();
 
-        for (idx, point, is_bf) in src_points
-            .into_iter()
-            .zip(tgt_points.into_iter())
-            .enumerate()
-            .filter_map(|(i, (src, tgt))| {
-                points_cross_mesh(&self.mesh, &vec_to_point(src), &vec_to_point(tgt))
-                    .map(|o| (i, o.0, o.1))
+            for (idx, point, is_bf) in src_points
+                .into_iter()
+                .zip(tgt_points.into_iter())
+                .enumerate()
+                .filter_map(|(i, (src, tgt))| {
+                    points_cross_mesh(&self.mesh, &vec_to_point(src), &vec_to_point(tgt))
+                        .map(|o| (i, o.0, o.1))
+                })
+            {
+                idxs.push(idx);
+                intersections.push(point_to_vec(&point));
+                is_backface.push(is_bf);
+            }
+
+            (idxs, intersections, is_backface)
+        })
+    }
+
+    pub fn intersections_many_threaded(
+        &self,
+        py: Python,
+        src_points: Vec<Vec<Precision>>,
+        tgt_points: Vec<Vec<Precision>>,
+        threads: usize,
+    ) -> (Vec<usize>, Vec<Vec<Precision>>, Vec<bool>) {
+        py.allow_threads(|| {
+            let pool = rayon::ThreadPoolBuilder::new()
+                .num_threads(threads)
+                .build()
+                .unwrap();
+            pool.install(|| {
+                let (idxs, (intersections, is_backface)) = src_points
+                    .into_par_iter()
+                    .zip(tgt_points.into_par_iter())
+                    .enumerate()
+                    .filter_map(|(i, (src, tgt))| {
+                        points_cross_mesh(&self.mesh, &vec_to_point(src), &vec_to_point(tgt))
+                            .map(|o| (i, (point_to_vec(&o.0), o.1)))
+                    })
+                    .unzip();
+
+                (idxs, intersections, is_backface)
             })
-        {
-            idxs.push(idx);
-            intersections.push(point_to_vec(&point));
-            is_backface.push(is_bf);
-        }
-
-        (idxs, intersections, is_backface)
+        })
     }
 }
 
